"""
exobrain-actions-client

This package provides a REST API client for interacting with the Exobrain Actions API.

Key features:

- Access to various API resources such as version status and health checks (ping).
- Retrieval, triggering, or updating of action data specific to a service or organization.
- Standardized HTTP calls to different API endpoints (version, health, action data, calculation triggers, KPI updates).
"""

__version__ = "0.1.1"
__release__ = "2025-07-15"
__author__ = "Laurent LAPORTE <laurent.laporte.pro@gmail.com>"
__copyright__ = f"Copyright 2025 {__author__}"
